﻿using RimWorld;
using RimWorld.Planet;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Verse;

namespace FinishingTouchesByTheBestBuilder
{
    public class DataContext : WorldComponent
    {
        public static DataContext current = null;

        public bool enabled = false;
        public int choiceBestPawnOptionPriority = 3;
        public bool notBuildInspireCheck;
        public int addLevelByInspire = 4;
        public int addLevelByRoleEffect = 2;

        public SettingDialog settingDialog = new SettingDialog();
        public Dictionary<int, PawnState> bestPawnStates = new Dictionary<int, PawnState>();
        public Dictionary<int, int> bestPawnLastRefreshTicks = new Dictionary<int, int>();
        public Dictionary<int, HashSet<PawnState>> pawnStates = new Dictionary<int, HashSet<PawnState>>();
        public DataContext(World world) : base(world)
        {
            current = this;
        }

        public static void UpdatePawnState(Pawn pawn, WorkTypeDef def)
        {
            if (def == WorkTypeDefOf.Construction)
            {
                UpdatePawnState(pawn);
                DataContext.current.RefreshPawnStates();
            }
        }
        public static void UpdatePawnState(Pawn pawn)
        {
            if (pawn == null || !CommonHelper.ValidPawn(pawn))
            {
                return;
            }
            int mapId = pawn.Map.uniqueID;
            PawnState bestPawnState = null;
            if (!current.bestPawnStates.TryGetValue(mapId, out bestPawnState))
            {
                current.bestPawnStates[mapId] = null;
            }
            HashSet<PawnState> states;
            if (!current.pawnStates.TryGetValue(mapId, out states))
            {
                states = new HashSet<PawnState>();
                current.pawnStates[mapId] = states;
            }
            int skill;
            PawnState pawnState = states.FirstOrDefault(x => x.pawn == pawn);
            if (pawnState == null)
            {
                pawnState = new PawnState(pawn);
                states.Add(pawnState);
                skill = pawnState.skill;
            }
            else
            {
                skill = pawnState.RefreshSkill();
            }
            int pawnPriority = CommonHelper.GetSettingPriority(pawn);
            if (pawnPriority > 0 && pawnPriority <= current.choiceBestPawnOptionPriority)
            {
                if (bestPawnState == null || skill > bestPawnState.skill)
                {
                    current.bestPawnStates[mapId] = pawnState;
                }
            }
        }

        public void ValidateAndRefreshPawnStates()
        {
            //Pawn設定の妥当性チェック＆削除
            foreach (KeyValuePair<int, HashSet<PawnState>> states in pawnStates)
            {
                Queue<PawnState> removables = new Queue<PawnState>(states.Value.Where(x => !CommonHelper.ValidPawn(x.pawn) || states.Key != x.pawn.Map.uniqueID));
                while (removables.Any())
                {
                    PawnState removable = removables.Dequeue();
                    if (this.bestPawnStates.TryGetValue(states.Key, out PawnState pawnState) && pawnState != null)
                    {
                        if (pawnState.pawn == removable.pawn)
                        {
                            this.bestPawnStates[states.Key] = null;
                        }
                    }
                    states.Value.Remove(removable);
                }
            }

            //残存Pawnのスキル更新
            RefreshPawnStates();
        }

        void RefreshPawnStates()
        {
            DataContext dc = DataContext.current;
            foreach (KeyValuePair<int, HashSet<PawnState>> states in pawnStates)
            {
                int mapId = states.Key;
                this.bestPawnStates.TryGetValue(mapId, out PawnState bestPawn);

                int bestPawnLevel = -1, bestPawnPriority = int.MaxValue;
                if (bestPawn?.pawn != null && (!notBuildInspireCheck || !CommonHelper.Inspired(bestPawn.pawn)))
                {
                    bestPawnPriority = CommonHelper.GetSettingPriority(bestPawn.pawn);
                    if (bestPawnPriority <= dc.choiceBestPawnOptionPriority)
                    {
                        bestPawnLevel = bestPawn.RefreshSkill();
                    }
                    else
                    {
                        bestPawnPriority = int.MaxValue;
                    }
                }
                foreach (PawnState state in states.Value)
                {
                    if (bestPawnLevel >= 0 && state == bestPawn)
                    {
                        continue;
                    }
                    int pawnSkill = -1, pawnPriority = int.MaxValue;
                    if (state?.pawn == null || (notBuildInspireCheck && CommonHelper.Inspired(state.pawn)))
                    {
                        continue;
                    }
                    pawnPriority = CommonHelper.GetSettingPriority(state.pawn);
                    if (pawnPriority > dc.choiceBestPawnOptionPriority)
                    {
                        continue;
                    }
                    pawnSkill = state.RefreshSkill();
                    if (pawnSkill > bestPawnLevel)
                    {
                        bestPawnLevel = pawnSkill;
                        bestPawn = state;
                    }
                    else if (pawnSkill == bestPawnLevel && pawnPriority < bestPawnPriority)
                    {
                        bestPawnLevel = pawnSkill;
                        bestPawn = state;
                    }
                }
                if (bestPawnLevel < 0)
                {
                    this.bestPawnStates[mapId] = null;
                }
                else
                {
                    this.bestPawnStates[mapId] = bestPawn;
                }
            }
        }

        public override void WorldComponentTick()
        {
            base.WorldComponentTick();
            if (Find.TickManager.TicksGame % CommonHelper.TICK_INTERVAL_CONTEXT == 0)
            {
                ValidateAndRefreshPawnStates();
            }
        }

        public override void ExposeData()
        {
            base.ExposeData();
            Scribe_Values.Look<bool>(ref enabled, "enabled", false);
            Scribe_Values.Look<int>(ref choiceBestPawnOptionPriority, "choiceBestPawnOptionPriority", 3);
            Scribe_Values.Look<bool>(ref notBuildInspireCheck, "notBuildInspireCheck", false);
            Scribe_Values.Look<int>(ref addLevelByInspire, "addLevelByInspire", 4);
            Scribe_Values.Look<int>(ref addLevelByRoleEffect, "addLevelByRoleEffect", 2);
        }
    }
}
